@@ -973,7 +973,18 @@ update_cells_row (ChafaCanvas *canvas, gint row)
              * sure the "blank" char has no foreground features. It's safest to
              * permit this optimization only with ASCII space. */
             if (canvas->blank_char == ' ' && cx > 0)
+            {
                 cells [cx].fg_color = cells [cx - 1].fg_color;
+
+                /* We may use inverted colors when the foreground is transparent.
+                 * Some downstream tools don't handle this and will keep
+                 * modulating the wrong pen. In order to suppress long runs of
+                 * artifacts, make the (unused) foreground pen opaque (gh#108). */
+                if (canvas->config.canvas_mode == CHAFA_CANVAS_MODE_TRUECOLOR)
+                    cells [cx].fg_color |= 0xff000000;
+                else if (cells [cx].fg_color == CHAFA_PALETTE_INDEX_TRANSPARENT)
+                    cells [cx].fg_color = CHAFA_PALETTE_INDEX_FG;
+            }
         }
     }
 }
@@ -1561,6 +1572,11 @@ chafa_canvas_draw_all_pixels (ChafaCanvas *canvas, ChafaPixelType src_pixel_type
     }
     else if (canvas->config.pixel_mode == CHAFA_PIXEL_MODE_KITTY)
     {
+        ChafaColor bg_color;
+
+        chafa_unpack_color (canvas->config.bg_color_packed_rgb, &bg_color);
+        bg_color.ch [3] = canvas->config.alpha_threshold < 1 ? 0x00 : 0xff;
+
         /* Kitty mode */
 
         canvas->fg_palette.alpha_threshold = canvas->config.alpha_threshold;
@@ -1570,7 +1586,8 @@ chafa_canvas_draw_all_pixels (ChafaCanvas *canvas, ChafaPixelType src_pixel_type
                                             src_pixel_type,
                                             src_pixels,
                                             src_width, src_height,
-                                            src_rowstride);
+                                            src_rowstride,
+                                            bg_color);
     }
     else  /* if (canvas->config.pixel_mode == CHAFA_PIXEL_MODE_ITERM2) */
     {
