@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2019-2021 Hans Petter Jansson
+/* Copyright (C) 2019-2022 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that turns images into character art.
  *
@@ -20,6 +20,9 @@
 #ifndef __CHAFA_BITFIELD_H__
 #define __CHAFA_BITFIELD_H__
 
+#include <string.h>
+#include <glib.h>
+
 G_BEGIN_DECLS
 
 typedef struct
