@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2020-2021 Hans Petter Jansson
+/* Copyright (C) 2020-2022 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that turns images into character art.
  *
@@ -30,6 +30,7 @@ typedef struct
     gunichar cur_char;
     gint n_reps;
     guint cur_inverted : 1;
+    guint cur_bold : 1;
     guint32 cur_fg;
     guint32 cur_bg;
 
@@ -39,221 +40,6 @@ typedef struct
 }
 PrintCtx;
 
-#define CODE_POINT_MAX 0x100000
-
-typedef struct
-{
-    gunichar c;
-    gint count;
-}
-CharStat;
-
-static gint
-compare_char_stats (gconstpointer a, gconstpointer b)
-{
-    const CharStat *acs = a;
-    const CharStat *bcs = b;
-
-    return acs->count - bcs->count;
-}
-
-static gboolean
-find_symbol (const ChafaSymbolMap *symbol_map, gunichar c,
-             const ChafaSymbol **sym_out, const ChafaSymbol2 **sym2_out)
-{
-    gint i;
-
-    *sym_out = NULL;
-    *sym2_out = NULL;
-
-    for (i = 0; i < symbol_map->n_symbols; i++)
-    {
-        const ChafaSymbol *sym = &symbol_map->symbols [i];
-
-        if (sym->c == c)
-        {
-            *sym_out = sym;
-            return TRUE;
-        }
-    }
-
-    for (i = 0; i < symbol_map->n_symbols2; i++)
-    {
-        const ChafaSymbol2 *sym = &symbol_map->symbols2 [i];
-
-        if (sym->sym [0].c == c)
-        {
-            *sym2_out = sym;
-            return TRUE;
-        }
-    }
-
-    return FALSE;
-}
-
-static void
-dump_coverage (const ChafaSymbol *sym)
-{
-    gint i;
-
-    g_printerr ("        CHAFA_SYMBOL_OUTLINE_%uX%u (\n",
-                CHAFA_SYMBOL_WIDTH_PIXELS,
-                CHAFA_SYMBOL_HEIGHT_PIXELS);
-
-    for (i = 0; ; i++)
-    {
-        if (i % CHAFA_SYMBOL_WIDTH_PIXELS == 0)
-            g_printerr ("            \"");
-
-        if (sym->coverage [i] == 0)
-            g_printerr (" ");
-        else
-            g_printerr ("X");
-
-        if (i == CHAFA_SYMBOL_N_PIXELS - 1)
-            break;
-
-        if (i % CHAFA_SYMBOL_WIDTH_PIXELS == CHAFA_SYMBOL_WIDTH_PIXELS - 1)
-            g_printerr ("\"\n");
-    }
-
-    g_printerr ("\")\n");
-}
-
-static void
-dump_coverage2 (const ChafaSymbol2 *sym)
-{
-    gint i;
-
-    g_printerr ("        CHAFA_SYMBOL_OUTLINE_16X8 (\n");
-
-    for (i = 0; ; i++)
-    {
-        if (i % 16 == 0)
-            g_printerr ("            \"");
-
-        if (i % 16 < 8)
-        {
-            if (sym->sym [0].coverage [((i / 16) * 8) + (i % 16)] == 0)
-                g_printerr (" ");
-            else
-                g_printerr ("X");
-        }
-        else
-        {
-            if (sym->sym [1].coverage [((i / 16) * 8) + (i % 16) - 8] == 0)
-                g_printerr (" ");
-            else
-                g_printerr ("X");
-        }
-
-        if (i == 127)
-            break;
-
-        if (i % 16 == 15)
-            g_printerr ("\"\n");
-    }
-
-    g_printerr ("\")\n");
-}
-
-static void
-dump_symbol (ChafaCanvas *canvas, const CharStat *stat)
-{
-    const ChafaSymbol *sym;
-    const ChafaSymbol2 *sym2;
-    gchar buf [8];
-    gint len;
-
-    if (!find_symbol (&canvas->config.symbol_map, stat->c, &sym, &sym2))
-        return;
-
-    len = g_unichar_to_utf8 (stat->c, buf);
-    if (!len)
-        return;
-    buf [len] = '\0';
-
-    g_printerr (
-        "    {\n"
-        "        /* [%s] freq=%d */\n"
-        "        CHAFA_SYMBOL_TAG_NONE,\n"
-        "        0x%x,\n",
-        buf, stat->count,
-        stat->c);
-
-    if (sym)
-    {
-        dump_coverage (sym);
-    }
-    else if (sym2)
-    {
-        dump_coverage2 (sym2);
-    }
-
-    g_printerr ("    },\n");
-}
-
-static void
-dump_stats (ChafaCanvas *canvas)
-{
-    CharStat *stats;
-    gint i, i_max;
-
-    stats = g_new (CharStat, CODE_POINT_MAX);
-
-    for (i = 0; i < CODE_POINT_MAX; i++)
-    {
-        stats [i].c = i;
-        stats [i].count = 0;
-    }
-
-    i_max = canvas->config.width * canvas->config.height;
-
-    for (i = 0; i < i_max; i++)
-    {
-        ChafaCanvasCell *cell = &canvas->cells [i];
-
-        /* Wide symbols have a zero code point in the rightmost cell */
-        if (cell->c == 0)
-            continue;
-
-        if (cell->c >= CODE_POINT_MAX)
-            continue;
-
-        stats [cell->c].count++;
-    }
-
-#if 0
-    qsort (stats, CODE_POINT_MAX, sizeof (CharStat), compare_char_stats);
-
-    for (i = 0; i < CODE_POINT_MAX; i++)
-    {
-        CharStat *stat = &stats [i];
-
-        if (stat->count < 1)
-            continue;
-
-        dump_symbol (canvas, stat);
-    }
-#elif 0
-    for (i = 0x30a0; i < 0x30ff; i++)
-    {
-        CharStat *stat = &stats [i];
-
-        dump_symbol (canvas, stat);
-    }
-#elif 0
-    for (i = 0xff65; i < 0xffa0; i++)
-    {
-        CharStat *stat = &stats [i];
-
-        dump_symbol (canvas, stat);
-    }
-#endif
-
-    g_free (stats);
-}
-
 static gint
 cmp_colors (ChafaColor a, ChafaColor b)
 {
@@ -330,6 +116,7 @@ reset_attributes (PrintCtx *ctx, gchar *out)
     out = chafa_term_info_emit_reset_attributes (ctx->term_info, out);
 
     ctx->cur_inverted = FALSE;
+    ctx->cur_bold = FALSE;
     ctx->cur_fg = CHAFA_PALETTE_INDEX_TRANSPARENT;
     ctx->cur_bg = CHAFA_PALETTE_INDEX_TRANSPARENT;
     ctx->cur_fg_direct.ch [3] = 0;
@@ -454,15 +241,19 @@ emit_ansi_truecolor (PrintCtx *ctx, gchar *out, gint i, gint i_max)
 }
 
 G_GNUC_WARN_UNUSED_RESULT static gchar *
-handle_inverted_with_reuse (PrintCtx *ctx, gchar *out,
-                            guint32 fg, guint32 bg, gboolean inverted)
+handle_attrs_with_reuse (PrintCtx *ctx, gchar *out,
+                         guint32 fg, guint32 bg,
+                         gboolean inverted, gboolean bold)
 {
-    /* We must check fg_only_enabled because we can run into the situation where
-     * fg is set to transparent. */
-    if (!ctx->canvas->config.fg_only_enabled
-        && ((ctx->cur_inverted && !inverted)
-            || (ctx->cur_fg != CHAFA_PALETTE_INDEX_TRANSPARENT && fg == CHAFA_PALETTE_INDEX_TRANSPARENT)
-            || (ctx->cur_bg != CHAFA_PALETTE_INDEX_TRANSPARENT && bg == CHAFA_PALETTE_INDEX_TRANSPARENT)))
+    /* In FG-only mode, we can't use inverse color or bold because the
+     * attribute reset would mess with the background color. */
+    if (ctx->canvas->config.fg_only_enabled)
+        return out;
+
+    if ((ctx->cur_inverted && !inverted)
+        || (ctx->cur_bold && !bold)
+        || (ctx->cur_fg != CHAFA_PALETTE_INDEX_TRANSPARENT && fg == CHAFA_PALETTE_INDEX_TRANSPARENT)
+        || (ctx->cur_bg != CHAFA_PALETTE_INDEX_TRANSPARENT && bg == CHAFA_PALETTE_INDEX_TRANSPARENT))
     {
         out = flush_chars (ctx, out);
         out = reset_attributes (ctx, out);
@@ -474,6 +265,12 @@ handle_inverted_with_reuse (PrintCtx *ctx, gchar *out,
         out = chafa_term_info_emit_invert_colors (ctx->term_info, out);
     }
 
+    if (!ctx->cur_bold && bold)
+    {
+        out = flush_chars (ctx, out);
+        out = chafa_term_info_emit_enable_bold (ctx->term_info, out);
+    }
+
     return out;
 }
 
@@ -483,7 +280,7 @@ emit_attributes_256 (PrintCtx *ctx, gchar *out,
 {
     if (ctx->canvas->config.optimizations & CHAFA_OPTIMIZATION_REUSE_ATTRIBUTES)
     {
-        out = handle_inverted_with_reuse (ctx, out, fg, bg, inverted);
+        out = handle_attrs_with_reuse (ctx, out, fg, bg, inverted, FALSE);
 
         if (fg != ctx->cur_fg)
         {
@@ -575,7 +372,7 @@ emit_attributes_16 (PrintCtx *ctx, gchar *out,
 {
     if (ctx->canvas->config.optimizations & CHAFA_OPTIMIZATION_REUSE_ATTRIBUTES)
     {
-        out = handle_inverted_with_reuse (ctx, out, fg, bg, inverted);
+        out = handle_attrs_with_reuse (ctx, out, fg, bg, inverted, FALSE);
 
         if (fg != ctx->cur_fg)
         {
@@ -662,6 +459,102 @@ emit_ansi_16 (PrintCtx *ctx, gchar *out, gint i, gint i_max)
     return out;
 }
 
+G_GNUC_WARN_UNUSED_RESULT static gchar *
+emit_attributes_16_8 (PrintCtx *ctx, gchar *out,
+                          guint32 fg, guint32 bg, gboolean inverted)
+{
+    if (ctx->canvas->config.optimizations & CHAFA_OPTIMIZATION_REUSE_ATTRIBUTES)
+    {
+        out = handle_attrs_with_reuse (ctx, out, fg, bg, inverted, fg > 7 && fg < 256 ? TRUE : FALSE);
+
+        if (fg != ctx->cur_fg)
+        {
+            if (bg != ctx->cur_bg && bg != CHAFA_PALETTE_INDEX_TRANSPARENT)
+            {
+                out = flush_chars (ctx, out);
+                out = chafa_term_info_emit_set_color_fgbg_8 (ctx->term_info, out, fg & 7, bg);
+            }
+            else if (fg != CHAFA_PALETTE_INDEX_TRANSPARENT)
+            {
+                out = flush_chars (ctx, out);
+                out = chafa_term_info_emit_set_color_fg_8 (ctx->term_info, out, fg & 7);
+            }
+        }
+        else if (bg != ctx->cur_bg && bg != CHAFA_PALETTE_INDEX_TRANSPARENT)
+        {
+            out = flush_chars (ctx, out);
+            out = chafa_term_info_emit_set_color_bg_8 (ctx->term_info, out, bg);
+        }
+    }
+    else
+    {
+        out = flush_chars (ctx, out);
+        out = reset_attributes (ctx, out);
+        if (inverted)
+            out = chafa_term_info_emit_invert_colors (ctx->term_info, out);
+        if (fg > 7)
+            out = chafa_term_info_emit_enable_bold (ctx->term_info, out);
+
+        if (fg != CHAFA_PALETTE_INDEX_TRANSPARENT)
+        {
+            if (bg != CHAFA_PALETTE_INDEX_TRANSPARENT)
+            {
+                out = chafa_term_info_emit_set_color_fgbg_8 (ctx->term_info, out, fg & 7, bg);
+            }
+            else
+            {
+                out = chafa_term_info_emit_set_color_fg_8 (ctx->term_info, out, fg & 7);
+            }
+        }
+        else if (bg != CHAFA_PALETTE_INDEX_TRANSPARENT)
+        {
+            out = chafa_term_info_emit_set_color_bg_8 (ctx->term_info, out, bg);
+        }
+    }
+
+    ctx->cur_fg = fg;
+    ctx->cur_bg = bg;
+    ctx->cur_inverted = inverted;
+    ctx->cur_bold = fg > 7 && fg < 256 ? TRUE : FALSE;
+    return out;
+}
+
+/* Uses bold for bright FG colors. */
+G_GNUC_WARN_UNUSED_RESULT static gchar *
+emit_ansi_16_8 (PrintCtx *ctx, gchar *out, gint i, gint i_max)
+{
+    for ( ; i < i_max; i++)
+    {
+        ChafaCanvasCell *cell = &ctx->canvas->cells [i];
+        guint32 fg, bg;
+
+        /* Wide symbols have a zero code point in the rightmost cell */
+        if (cell->c == 0)
+            continue;
+
+        fg = cell->fg_color;
+        bg = cell->bg_color;
+
+        if (fg == CHAFA_PALETTE_INDEX_TRANSPARENT && bg != CHAFA_PALETTE_INDEX_TRANSPARENT)
+            out = emit_attributes_16_8 (ctx, out, bg, fg, TRUE);
+        else
+            out = emit_attributes_16_8 (ctx, out, fg, bg, FALSE);
+
+        if (fg == CHAFA_PALETTE_INDEX_TRANSPARENT && bg == CHAFA_PALETTE_INDEX_TRANSPARENT)
+        {
+            out = queue_char (ctx, out, ' ');
+            if (i < i_max - 1 && ctx->canvas->cells [i + 1].c == 0)
+                out = queue_char (ctx, out, ' ');
+        }
+        else
+        {
+            out = queue_char (ctx, out, cell->c);
+        }
+    }
+
+    return out;
+}
+
 G_GNUC_WARN_UNUSED_RESULT static gchar *
 emit_ansi_fgbg_bgfg (PrintCtx *ctx, gchar *out, gint i, gint i_max)
 {
@@ -808,6 +701,9 @@ build_ansi_gstring (ChafaCanvas *canvas, ChafaTermInfo *ti)
             case CHAFA_CANVAS_MODE_INDEXED_16:
                 out = emit_ansi_16 (&ctx, out, i, i_next);
                 break;
+            case CHAFA_CANVAS_MODE_INDEXED_16_8:
+                out = emit_ansi_16_8 (&ctx, out, i, i_next);
+                break;
             case CHAFA_CANVAS_MODE_INDEXED_8:
                 out = emit_ansi_16 (&ctx, out, i, i_next);
                 break;
@@ -849,9 +745,5 @@ chafa_canvas_print_symbols (ChafaCanvas *canvas, ChafaTermInfo *ti)
     g_assert (canvas != NULL);
     g_assert (ti != NULL);
 
-#if 0
-    dump_stats (canvas);
-#endif
-
     return build_ansi_gstring (canvas, ti);
 }
