@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2019-2021 Hans Petter Jansson
+/* Copyright (C) 2019-2022 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that turns images into character art.
  *
@@ -20,13 +20,312 @@
 #include "config.h"
 
 #include <stdlib.h>  /* abs */
+#include <string.h>  /* memcpy, memset */
 #include <math.h>  /* pow, cbrt, log, sqrt, atan2, cos, sin */
 #include "chafa.h"
 #include "internal/chafa-private.h"
 
 #define DEBUG(x)
 
-/* FIXME: Refactor old color selection code into a common palette framework */
+/* ---------------- *
+ * Color candidates *
+ * ---------------- */
+
+/* Some situations (like fill symbols) call for both a best and a second-best
+ * match. ChafaColorCandidates is used to track and return these. */
+
+static void
+init_candidates (ChafaColorCandidates *candidates)
+{
+    candidates->index [0] = candidates->index [1] = -1;
+    candidates->error [0] = candidates->error [1] = G_MAXINT;
+}
+
+static gboolean
+update_candidates (ChafaColorCandidates *candidates, gint index, gint error)
+{
+    if (error < candidates->error [0])
+    {
+        candidates->index [1] = candidates->index [0];
+        candidates->index [0] = index;
+        candidates->error [1] = candidates->error [0];
+        candidates->error [0] = error;
+        return TRUE;
+    }
+    else if (error < candidates->error [1])
+    {
+        candidates->index [1] = index;
+        candidates->error [1] = error;
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
+/* -------------------- *
+ * Fixed system palette *
+ * -------------------- */
+
+static const guint32 term_colors_256 [CHAFA_PALETTE_INDEX_MAX] =
+{
+    /* First 16 colors; these are usually set by the terminal and can vary quite a
+     * bit. We try to strike a balance. */
+
+    0x000000, 0x800000, 0x007000, 0x707000, 0x000070, 0x700070, 0x007070, 0xc0c0c0,
+    0x404040, 0xff0000, 0x00ff00, 0xffff00, 0x0000ff, 0xff00ff, 0x00ffff, 0xffffff,
+
+    /* 240 universal colors; a 216-entry color cube followed by 24 grays */
+
+    0x000000, 0x00005f, 0x000087, 0x0000af, 0x0000d7, 0x0000ff, 0x005f00, 0x005f5f,
+    0x005f87, 0x005faf, 0x005fd7, 0x005fff, 0x008700, 0x00875f, 0x008787, 0x0087af,
+
+    0x0087d7, 0x0087ff, 0x00af00, 0x00af5f, 0x00af87, 0x00afaf, 0x00afd7, 0x00afff,
+    0x00d700, 0x00d75f, 0x00d787, 0x00d7af, 0x00d7d7, 0x00d7ff, 0x00ff00, 0x00ff5f,
+
+    0x00ff87, 0x00ffaf, 0x00ffd7, 0x00ffff, 0x5f0000, 0x5f005f, 0x5f0087, 0x5f00af,
+    0x5f00d7, 0x5f00ff, 0x5f5f00, 0x5f5f5f, 0x5f5f87, 0x5f5faf, 0x5f5fd7, 0x5f5fff,
+
+    0x5f8700, 0x5f875f, 0x5f8787, 0x5f87af, 0x5f87d7, 0x5f87ff, 0x5faf00, 0x5faf5f,
+    0x5faf87, 0x5fafaf, 0x5fafd7, 0x5fafff, 0x5fd700, 0x5fd75f, 0x5fd787, 0x5fd7af,
+
+    0x5fd7d7, 0x5fd7ff, 0x5fff00, 0x5fff5f, 0x5fff87, 0x5fffaf, 0x5fffd7, 0x5fffff,
+    0x870000, 0x87005f, 0x870087, 0x8700af, 0x8700d7, 0x8700ff, 0x875f00, 0x875f5f,
+
+    0x875f87, 0x875faf, 0x875fd7, 0x875fff, 0x878700, 0x87875f, 0x878787, 0x8787af,
+    0x8787d7, 0x8787ff, 0x87af00, 0x87af5f, 0x87af87, 0x87afaf, 0x87afd7, 0x87afff,
+
+    0x87d700, 0x87d75f, 0x87d787, 0x87d7af, 0x87d7d7, 0x87d7ff, 0x87ff00, 0x87ff5f,
+    0x87ff87, 0x87ffaf, 0x87ffd7, 0x87ffff, 0xaf0000, 0xaf005f, 0xaf0087, 0xaf00af,
+
+    0xaf00d7, 0xaf00ff, 0xaf5f00, 0xaf5f5f, 0xaf5f87, 0xaf5faf, 0xaf5fd7, 0xaf5fff,
+    0xaf8700, 0xaf875f, 0xaf8787, 0xaf87af, 0xaf87d7, 0xaf87ff, 0xafaf00, 0xafaf5f,
+
+    0xafaf87, 0xafafaf, 0xafafd7, 0xafafff, 0xafd700, 0xafd75f, 0xafd787, 0xafd7af,
+    0xafd7d7, 0xafd7ff, 0xafff00, 0xafff5f, 0xafff87, 0xafffaf, 0xafffd7, 0xafffff,
+
+    0xd70000, 0xd7005f, 0xd70087, 0xd700af, 0xd700d7, 0xd700ff, 0xd75f00, 0xd75f5f,
+    0xd75f87, 0xd75faf, 0xd75fd7, 0xd75fff, 0xd78700, 0xd7875f, 0xd78787, 0xd787af,
+
+    0xd787d7, 0xd787ff, 0xd7af00, 0xd7af5f, 0xd7af87, 0xd7afaf, 0xd7afd7, 0xd7afff,
+    0xd7d700, 0xd7d75f, 0xd7d787, 0xd7d7af, 0xd7d7d7, 0xd7d7ff, 0xd7ff00, 0xd7ff5f,
+
+    0xd7ff87, 0xd7ffaf, 0xd7ffd7, 0xd7ffff, 0xff0000, 0xff005f, 0xff0087, 0xff00af,
+    0xff00d7, 0xff00ff, 0xff5f00, 0xff5f5f, 0xff5f87, 0xff5faf, 0xff5fd7, 0xff5fff,
+
+    0xff8700, 0xff875f, 0xff8787, 0xff87af, 0xff87d7, 0xff87ff, 0xffaf00, 0xffaf5f,
+    0xffaf87, 0xffafaf, 0xffafd7, 0xffafff, 0xffd700, 0xffd75f, 0xffd787, 0xffd7af,
+
+    0xffd7d7, 0xffd7ff, 0xffff00, 0xffff5f, 0xffff87, 0xffffaf, 0xffffd7, 0xffffff,
+    0x080808, 0x121212, 0x1c1c1c, 0x262626, 0x303030, 0x3a3a3a, 0x444444, 0x4e4e4e,
+
+    0x585858, 0x626262, 0x6c6c6c, 0x767676, 0x808080, 0x8a8a8a, 0x949494, 0x9e9e9e,
+    0xa8a8a8, 0xb2b2b2, 0xbcbcbc, 0xc6c6c6, 0xd0d0d0, 0xdadada, 0xe4e4e4, 0xeeeeee,
+
+    /* Special colors */
+
+    0x808080,  /* Transparent */
+    0xffffff,  /* Terminal's default foreground */
+    0x000000,  /* Terminal's default background */
+};
+
+static ChafaPaletteColor fixed_palette_256 [CHAFA_PALETTE_INDEX_MAX];
+static guchar color_cube_216_channel_index [256];
+static gboolean palette_initialized;
+
+static const ChafaColor *
+get_fixed_palette_color (guint index, ChafaColorSpace color_space)
+{
+    return &fixed_palette_256 [index].col [color_space];
+}
+
+void
+chafa_init_palette (void)
+{
+    gint i;
+
+    if (palette_initialized)
+        return;
+
+    for (i = 0; i < CHAFA_PALETTE_INDEX_MAX; i++)
+    {
+        chafa_unpack_color (term_colors_256 [i], &fixed_palette_256 [i].col [0]);
+        chafa_color_rgb_to_din99d (&fixed_palette_256 [i].col [0], &fixed_palette_256 [i].col [1]);
+
+        fixed_palette_256 [i].col [0].ch [3] = 0xff;  /* Fully opaque */
+        fixed_palette_256 [i].col [1].ch [3] = 0xff;  /* Fully opaque */
+    }
+
+    /* Transparent color */
+    fixed_palette_256 [CHAFA_PALETTE_INDEX_TRANSPARENT].col [0].ch [3] = 0x00;
+    fixed_palette_256 [CHAFA_PALETTE_INDEX_TRANSPARENT].col [1].ch [3] = 0x00;
+
+    for (i = 0; i < 0x5f / 2; i++)
+        color_cube_216_channel_index [i] = 0;
+    for ( ; i < (0x5f + 0x87) / 2; i++)
+        color_cube_216_channel_index [i] = 1;
+    for ( ; i < (0x87 + 0xaf) / 2; i++)
+        color_cube_216_channel_index [i] = 2;
+    for ( ; i < (0xaf + 0xd7) / 2; i++)
+        color_cube_216_channel_index [i] = 3;
+    for ( ; i < (0xd7 + 0xff) / 2; i++)
+        color_cube_216_channel_index [i] = 4;
+    for ( ; i <= 0xff; i++)
+        color_cube_216_channel_index [i] = 5;
+
+    palette_initialized = TRUE;
+}
+
+static gint
+update_candidates_with_color_index_diff (ChafaColorCandidates *candidates, ChafaColorSpace color_space, const ChafaColor *color, gint index)
+{
+    const ChafaColor *palette_color;
+    gint error;
+
+    palette_color = get_fixed_palette_color (index, color_space);
+    error = chafa_color_diff_fast (color, palette_color);
+    update_candidates (candidates, index, error);
+
+    return error;
+}
+
+static void
+pick_color_fixed_216_cube (const ChafaColor *color, ChafaColorSpace color_space, ChafaColorCandidates *candidates)
+{
+    gint i;
+
+    g_assert (color_space == CHAFA_COLOR_SPACE_RGB);
+
+    i = 16 + (color_cube_216_channel_index [color->ch [0]] * 6 * 6
+              + color_cube_216_channel_index [color->ch [1]] * 6
+              + color_cube_216_channel_index [color->ch [2]]);
+
+    update_candidates_with_color_index_diff (candidates, color_space, color, i);
+}
+
+static void
+pick_color_fixed_24_grays (const ChafaColor *color, ChafaColorSpace color_space, ChafaColorCandidates *candidates)
+{
+    const ChafaColor *palette_color;
+    gint error, last_error = G_MAXINT;
+    gint step, i;
+
+    g_assert (color_space == CHAFA_COLOR_SPACE_RGB);
+
+    i = 232 + 12;
+    last_error = update_candidates_with_color_index_diff (candidates, color_space, color, i);
+
+    palette_color = get_fixed_palette_color (i + 1, color_space);
+    error = chafa_color_diff_fast (color, palette_color);
+    if (error < last_error)
+    {
+        update_candidates (candidates, i, error);
+        last_error = error;
+        step = 1;
+        i++;
+    }
+    else
+    {
+        step = -1;
+    }
+
+    do
+    {
+        i += step;
+        palette_color = get_fixed_palette_color (i, color_space);
+
+        error = chafa_color_diff_fast (color, palette_color);
+        if (error > last_error)
+            break;
+
+        update_candidates (candidates, i, error);
+        last_error = error;
+    }
+    while (i >= 232 && i <= 255);
+}
+
+static void
+pick_color_fixed_16 (const ChafaColor *color, ChafaColorSpace color_space, ChafaColorCandidates *candidates)
+{
+    gint i;
+
+    for (i = 0; i < 16; i++)
+    {
+        update_candidates_with_color_index_diff (candidates, color_space, color, i);
+    }
+}
+
+static void
+pick_color_fixed_8 (const ChafaColor *color, ChafaColorSpace color_space, ChafaColorCandidates *candidates)
+{
+    gint i;
+
+    for (i = 0; i < 8; i++)
+    {
+        update_candidates_with_color_index_diff (candidates, color_space, color, i);
+    }
+}
+
+static void
+pick_color_fixed_256 (const ChafaColor *color, ChafaColorSpace color_space, ChafaColorCandidates *candidates)
+{
+    gint i;
+
+    if (color_space == CHAFA_COLOR_SPACE_RGB)
+    {
+        pick_color_fixed_216_cube (color, color_space, candidates);
+        pick_color_fixed_24_grays (color, color_space, candidates);
+
+        /* Do this last so ties are broken in favor of high-index colors. */
+        pick_color_fixed_16 (color, color_space, candidates);
+    }
+    else
+    {
+        for (i = 0; i < 256; i++)
+        {
+            update_candidates_with_color_index_diff (candidates, color_space, color, i);
+        }
+    }
+}
+
+static void
+pick_color_fixed_240 (const ChafaColor *color, ChafaColorSpace color_space, ChafaColorCandidates *candidates)
+{
+    gint i;
+
+    if (color_space == CHAFA_COLOR_SPACE_RGB)
+    {
+        pick_color_fixed_216_cube (color, color_space, candidates);
+        pick_color_fixed_24_grays (color, color_space, candidates);
+    }
+    else
+    {
+        /* Check color cube, but not lower 16, bg or fg. Slow! */
+        for (i = 16; i < 256; i++)
+        {
+            update_candidates_with_color_index_diff (candidates, color_space, color, i);
+        }
+    }
+}
+
+/* Pick the best approximation of color from a palette consisting of
+ * fg_color and bg_color */
+static void
+pick_color_fixed_fgbg (const ChafaColor *color,
+                 const ChafaColor *fg_color, const ChafaColor *bg_color,
+                 ChafaColorCandidates *candidates)
+{
+    gint error;
+
+    error = chafa_color_diff_fast (color, fg_color);
+    update_candidates (candidates, CHAFA_PALETTE_INDEX_FG, error);
+
+    error = chafa_color_diff_fast (color, bg_color);
+    update_candidates (candidates, CHAFA_PALETTE_INDEX_BG, error);
+}
+
+/* --------------------------------- *
+ * Quantization for dynamic palettes *
+ * --------------------------------- */
 
 static gint
 find_dominant_channel (gconstpointer pixels, gint n_pixels)
@@ -475,6 +774,10 @@ clean_up (ChafaPalette *palette_out)
     }
 }
 
+/* --- *
+ * API *
+ * --- */
+
 void
 chafa_palette_init (ChafaPalette *palette_out, ChafaPaletteType type)
 {
@@ -482,41 +785,43 @@ chafa_palette_init (ChafaPalette *palette_out, ChafaPaletteType type)
 
     chafa_init_palette ();
     palette_out->type = type;
+    palette_out->transparent_index = CHAFA_PALETTE_INDEX_TRANSPARENT;
 
     for (i = 0; i < CHAFA_PALETTE_INDEX_MAX; i++)
     {
-        palette_out->colors [i].col [CHAFA_COLOR_SPACE_RGB] = *chafa_get_palette_color_256 (i, CHAFA_COLOR_SPACE_RGB);
-        palette_out->colors [i].col [CHAFA_COLOR_SPACE_DIN99D] = *chafa_get_palette_color_256 (i, CHAFA_COLOR_SPACE_DIN99D);
+        palette_out->colors [i].col [CHAFA_COLOR_SPACE_RGB] = *get_fixed_palette_color (i, CHAFA_COLOR_SPACE_RGB);
+        palette_out->colors [i].col [CHAFA_COLOR_SPACE_DIN99D] = *get_fixed_palette_color (i, CHAFA_COLOR_SPACE_DIN99D);
     }
 
-    palette_out->transparent_index = CHAFA_PALETTE_INDEX_TRANSPARENT;
+    switch (type)
+    {
+        case CHAFA_PALETTE_TYPE_FIXED_FGBG:
+            palette_out->first_color = CHAFA_PALETTE_INDEX_FG;
+            palette_out->n_colors = 2;
+            break;
 
-    palette_out->first_color = 0;
-    palette_out->n_colors = 256;
+        case CHAFA_PALETTE_TYPE_FIXED_8:
+            palette_out->n_colors = 8;
+            break;
 
-    if (type == CHAFA_PALETTE_TYPE_FIXED_240)
-    {
-        palette_out->first_color = 16;
-        palette_out->n_colors = 240;
-    }
-    else if (type == CHAFA_PALETTE_TYPE_FIXED_16)
-    {
-        palette_out->n_colors = 16;
-    }
-    else if (type == CHAFA_PALETTE_TYPE_FIXED_8)
-    {
-        palette_out->n_colors = 8;
-    }
-    else if (type == CHAFA_PALETTE_TYPE_FIXED_FGBG)
-    {
-        palette_out->first_color = CHAFA_PALETTE_INDEX_FG;
-        palette_out->n_colors = 2;
-    }
+        case CHAFA_PALETTE_TYPE_FIXED_16:
+            palette_out->n_colors = 16;
+            break;
 
-    if (palette_out->type == CHAFA_PALETTE_TYPE_DYNAMIC_256)
-    {
-        for (i = 0; i < CHAFA_COLOR_SPACE_MAX; i++)
-            chafa_color_table_init (&palette_out->table [i]);
+        case CHAFA_PALETTE_TYPE_FIXED_240:
+            palette_out->first_color = 16;
+            palette_out->n_colors = 240;
+            break;
+
+        case CHAFA_PALETTE_TYPE_FIXED_256:
+            palette_out->first_color = 0;
+            palette_out->n_colors = 256;
+            break;
+
+        case CHAFA_PALETTE_TYPE_DYNAMIC_256:
+            for (i = 0; i < CHAFA_COLOR_SPACE_MAX; i++)
+                chafa_color_table_init (&palette_out->table [i]);
+            break;
     }
 }
 
@@ -551,6 +856,7 @@ chafa_palette_copy (const ChafaPalette *src, ChafaPalette *dest)
 }
 
 /* pixels must point to RGBA8888 data to sample */
+/* FIXME: Rowstride etc? */
 void
 chafa_palette_generate (ChafaPalette *palette_out, gconstpointer pixels, gint n_pixels,
                         ChafaColorSpace color_space)
@@ -640,25 +946,26 @@ chafa_palette_lookup_nearest (const ChafaPalette *palette, ChafaColorSpace color
         if (!candidates)
             candidates = &candidates_temp;
 
-#if 0
-        /* Transparency */
+        init_candidates (candidates);
 
-        /* NOTE: Disabled because chafa_pick_color_*() deal
-         * with transparency */
         if (color->ch [3] < palette->alpha_threshold)
-            return palette->transparent_index;
-#endif
-
-        if (palette->type == CHAFA_PALETTE_TYPE_FIXED_256)
-            chafa_pick_color_256 (color, color_space, candidates);
+        {
+            /* Transparency */
+            candidates->index [0] = palette->transparent_index;
+            candidates->index [1] = palette->transparent_index;
+            candidates->error [0] = 0;
+            candidates->error [1] = 0;
+        }
+        else if (palette->type == CHAFA_PALETTE_TYPE_FIXED_256)
+            pick_color_fixed_256 (color, color_space, candidates);
         else if (palette->type == CHAFA_PALETTE_TYPE_FIXED_240)
-            chafa_pick_color_240 (color, color_space, candidates);
+            pick_color_fixed_240 (color, color_space, candidates);
         else if (palette->type == CHAFA_PALETTE_TYPE_FIXED_16)
-            chafa_pick_color_16 (color, color_space, candidates);
+            pick_color_fixed_16 (color, color_space, candidates);
         else if (palette->type == CHAFA_PALETTE_TYPE_FIXED_8)
-            chafa_pick_color_8 (color, color_space, candidates);
+            pick_color_fixed_8 (color, color_space, candidates);
         else /* CHAFA_PALETTE_TYPE_FIXED_FGBG */
-            chafa_pick_color_fgbg (color, color_space,
+            pick_color_fixed_fgbg (color,
                                    &palette->colors [CHAFA_PALETTE_INDEX_FG].col [color_space],
                                    &palette->colors [CHAFA_PALETTE_INDEX_BG].col [color_space],
                                    candidates);
@@ -694,9 +1001,9 @@ chafa_palette_lookup_with_error (const ChafaPalette *palette, ChafaColorSpace co
 
     if (error_inout)
     {
-        compensated_color.ch [0] = ((gint16) color.ch [0]) + ((error_inout->ch [0] * 0.9) / 16);
-        compensated_color.ch [1] = ((gint16) color.ch [1]) + ((error_inout->ch [1] * 0.9) / 16);
-        compensated_color.ch [2] = ((gint16) color.ch [2]) + ((error_inout->ch [2] * 0.9) / 16);
+        compensated_color.ch [0] = ((gint16) color.ch [0]) + ((error_inout->ch [0] * 0.9f) / 16);
+        compensated_color.ch [1] = ((gint16) color.ch [1]) + ((error_inout->ch [1] * 0.9f) / 16);
+        compensated_color.ch [2] = ((gint16) color.ch [2]) + ((error_inout->ch [2] * 0.9f) / 16);
 
         color.ch [0] = CLAMP (compensated_color.ch [0], 0, 255);
         color.ch [1] = CLAMP (compensated_color.ch [1], 0, 255);
