@@ -36,9 +36,6 @@
 #ifdef HAVE_TERMIOS_H
 # include <termios.h>  /* tcgetattr, tcsetattr */
 #endif
-#ifdef HAVE_WINDOWS_H
-# include <windows.h>
-#endif
 
 #include <glib/gstdio.h>
 
@@ -47,6 +44,14 @@
 #include "media-loader.h"
 #include "named-colors.h"
 
+/* Include after glib.h for G_OS_WIN32 */
+#ifdef G_OS_WIN32
+# ifdef HAVE_WINDOWS_H
+#  include <windows.h>
+# endif
+# include <io.h>
+#endif
+
 #define ANIM_FPS_MAX 100000.0
 #define FILE_DURATION_DEFAULT 0.0
 #define SCALE_MAX 9999.0
@@ -97,6 +102,7 @@ typedef struct
     guint32 bg_color;
     gboolean bg_color_set;
     gdouble transparency_threshold;
+    gboolean transparency_threshold_set;
     gdouble file_duration_s;
 
     /* If > 0.0, override the framerate specified by the input file. */
@@ -126,6 +132,15 @@ static TermSize detected_term_size;
 static gboolean using_detected_size = FALSE;
 static volatile sig_atomic_t interrupted_by_user = FALSE;
 
+#ifdef HAVE_TERMIOS_H
+static struct termios saved_termios;
+#endif
+
+#ifdef G_OS_WIN32
+static UINT saved_console_output_cp;
+static UINT saved_console_input_cp;
+#endif
+
 #ifdef HAVE_SIGACTION
 static void
 sigint_handler (G_GNUC_UNUSED int sig)
@@ -393,7 +408,7 @@ print_summary (void)
     "  -C, --center=BOOL  Center images [on, off]. Defaults to off.\n"
     "      --clear        Clear screen before processing each file.\n"
     "  -c, --colors=MODE  Set output color mode; one of [none, 2, 8, 16/8, 16, 240,\n"
-    "                     256, full]. Defaults to full (24-bit).\n"
+    "                     256, full]. Defaults to best guess.\n"
     "      --color-extractor=EXTR  Method for extracting color from an area\n"
     "                     [average, median]. Average is the default.\n"
     "      --color-space=CS  Color space used for quantization; one of [rgb, din99d].\n"
@@ -1229,15 +1244,6 @@ get_tty_size (TermSize *term_size_out)
     *term_size_out = term_size;
 }
 
-#ifdef HAVE_TERMIOS_H
-static struct termios saved_termios;
-#endif
-
-#ifdef G_OS_WIN32
-static UINT saved_console_output_cp;
-static UINT saved_console_input_cp;
-#endif
-
 static void
 tty_options_init (void)
 {
@@ -1637,6 +1643,8 @@ parse_options (int *argc, char **argv [])
 
     if (options.transparency_threshold == G_MAXDOUBLE)
         options.transparency_threshold = 0.5;
+    else
+        options.transparency_threshold_set = TRUE;
 
     if (options.transparency_threshold < 0.0 || options.transparency_threshold > 1.0)
     {
@@ -1827,7 +1835,8 @@ out:
 static GString *
 build_string (ChafaPixelType pixel_type, const guint8 *pixels,
               gint src_width, gint src_height, gint src_rowstride,
-              gint dest_width, gint dest_height)
+              gint dest_width, gint dest_height,
+              gboolean is_animation)
 {
     ChafaCanvasConfig *config;
     ChafaCanvas *canvas;
@@ -1848,8 +1857,13 @@ build_string (ChafaPixelType pixel_type, const guint8 *pixels,
     chafa_canvas_config_set_preprocessing_enabled (config, options.preprocess);
     chafa_canvas_config_set_fg_only_enabled (config, options.fg_only);
 
-    if (options.transparency_threshold >= 0.0)
+    if (is_animation
+        && options.pixel_mode == CHAFA_PIXEL_MODE_KITTY
+        && !options.transparency_threshold_set)
+        chafa_canvas_config_set_transparency_threshold (config, 1.0f);
+    else if (options.transparency_threshold >= 0.0)
         chafa_canvas_config_set_transparency_threshold (config, options.transparency_threshold);
+
     if (options.cell_width > 0 && options.cell_height > 0)
         chafa_canvas_config_set_cell_geometry (config, options.cell_width, options.cell_height);
 
@@ -1949,8 +1963,7 @@ run_generic (const gchar *filename, gboolean is_first_file, gboolean is_first_fr
         media_loader_goto_first_frame (media_loader);
 
         for (have_frame = TRUE;
-             (is_first_frame || is_animation)
-             && have_frame && !interrupted_by_user && (loop_n == 0 || anim_elapsed_s < options.file_duration_s);
+             have_frame && !interrupted_by_user && (loop_n == 0 || anim_elapsed_s < options.file_duration_s);
              have_frame = media_loader_goto_next_frame (media_loader))
         {
             gdouble elapsed_ms, remain_ms;
@@ -2005,7 +2018,8 @@ run_generic (const gchar *filename, gboolean is_first_file, gboolean is_first_fr
 
             gs = build_string (pixel_type, pixels,
                                src_width, src_height, src_rowstride,
-                               dest_width, dest_height);
+                               dest_width, dest_height,
+                               is_animation);
 
             p0 = buf;
 
@@ -2082,6 +2096,9 @@ run_generic (const gchar *filename, gboolean is_first_file, gboolean is_first_fr
             }
 
             is_first_frame = FALSE;
+
+            if (!is_animation)
+                break;
         }
 
         loop_n++;
@@ -2195,6 +2212,11 @@ proc_init (void)
     sigaction (SIGINT, &sa, NULL);
 #endif
 
+#ifdef G_OS_WIN32
+    setmode (fileno (stdin), O_BINARY);
+    setmode (fileno (stdout), O_BINARY);
+#endif
+
     /* Must do this early. Buffer size probably doesn't matter */
     setvbuf (stdout, NULL, _IOFBF, 32768);
 
