@@ -26,7 +26,11 @@
 #include <sys/stat.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdio.h>
+
+#ifdef G_OS_WIN32
+# include <stdio.h>
+# include <io.h>
+#endif
 
 #ifdef HAVE_MMAP
 # include <sys/mman.h>
@@ -51,9 +55,9 @@
  * flag is not always defined in our cross builds. */
 #ifndef O_BINARY
 # ifdef _O_BINARY
-# define O_BINARY _O_BINARY
+#  define O_BINARY _O_BINARY
 # else
-# define O_BINARY 0
+#  define O_BINARY 0
 # endif
 #endif
 
@@ -82,7 +86,7 @@ file_is_stdin (FileMapping *file_mapping)
     return file_mapping->path
         && file_mapping->path [0] == '-'
         && file_mapping->path [1] == '\0' ? TRUE : FALSE;
- }
+}
 
 static gsize
 safe_read (gint fd, void *buf, gsize len)
@@ -94,6 +98,7 @@ safe_read (gint fd, void *buf, gsize len)
    {
        unsigned int nread;
        int iread;
+       int saved_errno;
 
        /* Passing nread > INT_MAX to read is implementation defined in POSIX
         * 1003.1, therefore despite the unsigned argument portable code must
@@ -105,6 +110,7 @@ safe_read (gint fd, void *buf, gsize len)
            nread = (unsigned int)/*SAFE*/len;
 
        iread = read (fd, buffer, nread);
+       saved_errno = errno;
 
        if (iread == -1)
        {
@@ -112,7 +118,7 @@ safe_read (gint fd, void *buf, gsize len)
             * bytes read because of EINTR, yet it still returns -1 otherwise end
             * of file cannot be distinguished.
             */
-           if (errno != EINTR)
+           if (saved_errno != EINTR)
            {
                /* I.e. a permanent failure */
                return 0;
@@ -149,6 +155,7 @@ safe_write (gint fd, gconstpointer buf, gsize len)
     {
        guint to_write;
        gint n_written;
+       gint saved_errno;
 
        if (len > INT_MAX)
            to_write = INT_MAX;
@@ -156,10 +163,11 @@ safe_write (gint fd, gconstpointer buf, gsize len)
            to_write = (unsigned int) len;
 
        n_written = write (fd, buffer, to_write);
+       saved_errno = errno;
 
        if (n_written == -1)
        {
-           if (errno != EINTR)
+           if (saved_errno != EINTR)
                goto out;
        }
        else if (n_written < 0)
@@ -303,6 +311,10 @@ cache_stdin (FileMapping *file_mapping, GError **error)
     if (stdin_fd < 0)
         goto out;
 
+#ifdef G_OS_WIN32
+    setmode (stdin_fd, O_BINARY);
+#endif
+
     /* Read from stdin */
 
     buf = g_malloc (FILE_MEMORY_CACHE_MAX);
